@@ -56,7 +56,7 @@
 #define TRACKER_DB_PAGE_SIZE_DONT_SET -1
 
 /* Set current database version we are working with */
-#define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_0_9_21
+#define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_0_9_24
 #define TRACKER_DB_VERSION_FILE       "db-version.txt"
 #define TRACKER_DB_LOCALE_FILE        "db-locale.txt"
 
@@ -93,7 +93,8 @@ typedef enum {
 	TRACKER_DB_VERSION_0_9_15,  /* mtp:hidden */
 	TRACKER_DB_VERSION_0_9_16,  /* Fix for NB#184823 */
 	TRACKER_DB_VERSION_0_9_19,  /* collation */
-	TRACKER_DB_VERSION_0_9_21   /* Fix for NB#186055 */
+	TRACKER_DB_VERSION_0_9_21,  /* Fix for NB#186055 */
+	TRACKER_DB_VERSION_0_9_24   /* nmo:PhoneMessage class */
 } TrackerDBVersion;
 
 typedef struct {
@@ -141,7 +142,6 @@ static gboolean            db_exec_no_reply    (TrackerDBInterface *iface,
 static TrackerDBInterface *db_interface_create (TrackerDB           db);
 static TrackerDBInterface *tracker_db_manager_get_db_interfaces     (gint num, ...);
 static TrackerDBInterface *tracker_db_manager_get_db_interfaces_ro  (gint num, ...);
-static void                db_remove_version_file (void);
 static void                db_remove_locale_file  (void);
 
 static gboolean              initialized;
@@ -358,7 +358,9 @@ db_manager_remove_journal (void)
 			}
 
 			fullpath = g_build_filename (dirs[i], f, NULL);
-			g_unlink (fullpath);
+			if (g_unlink (fullpath) == -1) {
+				g_message ("%s", g_strerror (errno));
+			}
 			g_free (fullpath);
 		}
 
@@ -369,7 +371,9 @@ db_manager_remove_journal (void)
 	g_free (directory);
 
 	/* Remove active journal */
-	g_unlink (path);
+	if (g_unlink (path) == -1) {
+		g_message ("%s", g_strerror (errno));
+	}
 	g_free (path);
 }
 
@@ -410,15 +414,15 @@ db_manager_remove_all (gboolean rm_journal)
 
 		/* If also the journal is gone, we can also remove db-version.txt, it
 		 * would have no more relevance whatsoever. */
-		db_remove_version_file ();
+		tracker_db_manager_remove_version_file ();
 	}
 
 	/* Remove locale file also */
 	db_remove_locale_file ();
 }
 
-static void
-db_remove_version_file (void)
+void
+tracker_db_manager_remove_version_file (void)
 {
 	gchar *filename;
 
@@ -597,7 +601,8 @@ tracker_db_manager_set_current_locale (void)
 }
 
 static void
-db_manager_analyze (TrackerDB db)
+db_manager_analyze (TrackerDB           db,
+                    TrackerDBInterface *iface)
 {
 	guint64             current_mtime;
 
@@ -605,7 +610,7 @@ db_manager_analyze (TrackerDB db)
 
 	if (current_mtime > dbs[db].mtime) {
 		g_message ("  Analyzing DB:'%s'", dbs[db].name);
-		db_exec_no_reply (dbs[db].iface, "ANALYZE %s.Services", dbs[db].name);
+		db_exec_no_reply (iface, "ANALYZE %s.Services", dbs[db].name);
 
 		/* Remember current mtime for future */
 		dbs[db].mtime = current_mtime;
@@ -698,7 +703,9 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
                          gboolean              *first_time,
                          gboolean               shared_cache,
                          guint                  select_cache_size,
-                         guint                  update_cache_size)
+                         guint                  update_cache_size,
+                         TrackerBusyCallback    busy_callback,
+                         gpointer               busy_user_data)
 {
 	GType               etype;
 	TrackerDBVersion    version;
@@ -740,7 +747,7 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	old_flags = flags;
 
 	filename = g_strdup_printf ("tracker-%s", g_get_user_name ());
-        g_free (sys_tmp_dir);
+	g_free (sys_tmp_dir);
 	sys_tmp_dir = g_build_filename (g_get_tmp_dir (), filename, NULL);
 	g_free (filename);
 
@@ -754,7 +761,7 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 		sql_dir = g_strdup (env_path);
 	}
 
-        g_free (user_data_dir);
+	g_free (user_data_dir);
 	user_data_dir = g_build_filename (g_get_user_data_dir (),
 	                                  "tracker",
 	                                  "data",
@@ -788,9 +795,6 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	}
 
 	if (need_reindex) {
-		tracker_db_journal_init (NULL, FALSE);
-		db_manager_remove_journal ();
-		tracker_db_journal_shutdown ();
 		db_set_version ();
 	}
 
@@ -800,7 +804,7 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 		/* Fill absolute path for the database */
 		dir = location_to_directory (dbs[i].location);
 
-                g_free (dbs[i].abs_filename);
+		g_free (dbs[i].abs_filename);
 		dbs[i].abs_filename = g_build_filename (dir, dbs[i].file, NULL);
 
 		/* Check we have each database in place, if one is
@@ -909,6 +913,11 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 
 				loaded = TRUE;
 
+				tracker_db_interface_set_busy_handler (dbs[i].iface,
+				                                       busy_callback,
+				                                       "Integrity checking",
+				                                       busy_user_data);
+
 				stmt = tracker_db_interface_create_statement (dbs[i].iface, TRACKER_DB_STATEMENT_CACHE_TYPE_NONE, &error,
 				                                              "PRAGMA integrity_check(1)");
 
@@ -939,6 +948,8 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 						g_object_unref (cursor);
 					}
 				}
+
+				tracker_db_interface_set_busy_handler (dbs[i].iface, NULL, NULL, NULL);
 			}
 		}
 
@@ -969,7 +980,6 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 
 	if (!loaded) {
 		for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
-			dbs[i].iface = db_interface_create (i);
 			dbs[i].mtime = tracker_file_get_mtime (dbs[i].abs_filename);
 		}
 	}
@@ -1081,7 +1091,8 @@ void
 tracker_db_manager_move_to_temp (void)
 {
 	guint i;
-	gchar *cpath, *new_filename;
+	gchar *cpath, *filename;
+	gchar *fullpath;
 	gchar *directory, *rotate_to = NULL;
 	const gchar *dirs[3] = { NULL, NULL, NULL };
 	gsize chunk_size = 0;
@@ -1092,11 +1103,13 @@ tracker_db_manager_move_to_temp (void)
 	g_message ("Moving all database files");
 
 	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
-		new_filename = g_strdup_printf ("%s.tmp", dbs[i].abs_filename);
+		filename = g_strdup_printf ("%s.tmp", dbs[i].abs_filename);
 		g_message ("  Renaming database:'%s' -> '%s'",
-		           dbs[i].abs_filename, new_filename);
-		g_rename (dbs[i].abs_filename, new_filename);
-		g_free (new_filename);
+		           dbs[i].abs_filename, filename);
+		if (g_rename (dbs[i].abs_filename, filename) == -1) {
+			g_critical ("%s", g_strerror (errno));
+		}
+		g_free (filename);
 	}
 
 	cpath = g_strdup (tracker_db_journal_get_filename ());
@@ -1116,8 +1129,6 @@ tracker_db_manager_move_to_temp (void)
 		f_name = g_dir_read_name (journal_dir);
 
 		while (f_name) {
-			gchar *fullpath;
-
 			if (f_name) {
 				if (!g_str_has_prefix (f_name, TRACKER_DB_JOURNAL_FILENAME ".")) {
 					f_name = g_dir_read_name (journal_dir);
@@ -1126,9 +1137,11 @@ tracker_db_manager_move_to_temp (void)
 			}
 
 			fullpath = g_build_filename (dirs[i], f_name, NULL);
-			new_filename = g_strdup_printf ("%s.tmp", fullpath);
-			g_rename (fullpath, new_filename);
-			g_free (new_filename);
+			filename = g_strdup_printf ("%s.tmp", fullpath);
+			if (g_rename (fullpath, filename) == -1) {
+				g_critical ("%s", g_strerror (errno));
+			}
+			g_free (filename);
 			g_free (fullpath);
 			f_name = g_dir_read_name (journal_dir);
 		}
@@ -1136,15 +1149,25 @@ tracker_db_manager_move_to_temp (void)
 		g_dir_close (journal_dir);
 	}
 
+	fullpath = g_build_filename (directory, TRACKER_DB_JOURNAL_ONTOLOGY_FILENAME, NULL);
+	filename = g_strdup_printf ("%s.tmp", fullpath);
+	if (g_rename (fullpath, filename) == -1) {
+		g_critical ("%s", g_strerror (errno));
+	}
+	g_free (filename);
+	g_free (fullpath);
+
 	g_free (rotate_to);
 	g_free (directory);
 
-	new_filename = g_strdup_printf ("%s.tmp", cpath);
+	filename = g_strdup_printf ("%s.tmp", cpath);
 	g_message ("  Renaming journal:'%s' -> '%s'",
-	           cpath, new_filename);
-	g_rename (cpath, new_filename);
+	           cpath, filename);
+	if (g_rename (cpath, filename) == -1) {
+		g_critical ("%s", g_strerror (errno));
+	}
 	g_free (cpath);
-	g_free (new_filename);
+	g_free (filename);
 }
 
 
@@ -1152,7 +1175,8 @@ void
 tracker_db_manager_restore_from_temp (void)
 {
 	guint i;
-	gchar *cpath, *new_filename;
+	gchar *cpath, *filename;
+	gchar *fullpath;
 	gchar *directory, *rotate_to = NULL;
 	const gchar *dirs[3] = { NULL, NULL, NULL };
 	gsize chunk_size = 0;
@@ -1163,23 +1187,35 @@ tracker_db_manager_restore_from_temp (void)
 	g_message ("Moving all database files");
 
 	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
-		new_filename = g_strdup_printf ("%s.tmp", dbs[i].abs_filename);
+		filename = g_strdup_printf ("%s.tmp", dbs[i].abs_filename);
 		g_message ("  Renaming database:'%s' -> '%s'",
-		           dbs[i].abs_filename, new_filename);
-		g_rename (dbs[i].abs_filename, new_filename);
-		g_free (new_filename);
+		           dbs[i].abs_filename, filename);
+		if (g_rename (dbs[i].abs_filename, filename) == -1) {
+			g_critical ("%s", g_strerror (errno));
+		}
+		g_free (filename);
 	}
 
 	cpath = g_strdup (tracker_db_journal_get_filename ());
-	new_filename = g_strdup_printf ("%s.tmp", cpath);
+	filename = g_strdup_printf ("%s.tmp", cpath);
 	g_message ("  Renaming journal:'%s' -> '%s'",
-	           cpath, new_filename);
-	g_rename (cpath, new_filename);
-	g_free (new_filename);
+	           filename, cpath);
+	if (g_rename (filename, cpath) == -1) {
+		g_critical ("%s", g_strerror (errno));
+	}
+	g_free (filename);
 
 	directory = g_path_get_dirname (cpath);
 	tracker_db_journal_get_rotating (&do_rotate, &chunk_size, &rotate_to);
 
+	fullpath = g_build_filename (directory, TRACKER_DB_JOURNAL_ONTOLOGY_FILENAME, NULL);
+	filename = g_strdup_printf ("%s.tmp", fullpath);
+	if (g_rename (filename, fullpath) == -1) {
+		g_critical ("%s", g_strerror (errno));
+	}
+	g_free (filename);
+	g_free (fullpath);
+
 	dirs[0] = directory;
 	dirs[1] = do_rotate ? rotate_to : NULL;
 
@@ -1191,7 +1227,7 @@ tracker_db_manager_restore_from_temp (void)
 		f_name = g_dir_read_name (journal_dir);
 
 		while (f_name) {
-			gchar *fullpath, *ptr;
+			gchar *ptr;
 
 			if (f_name) {
 				if (!g_str_has_suffix (f_name, ".tmp")) {
@@ -1201,13 +1237,15 @@ tracker_db_manager_restore_from_temp (void)
 			}
 
 			fullpath = g_build_filename (dirs[i], f_name, NULL);
-			new_filename = g_strdup (fullpath);
-			ptr = strstr (new_filename, ".tmp");
+			filename = g_strdup (fullpath);
+			ptr = strstr (filename, ".tmp");
 			if (ptr) {
 				*ptr = '\0';
-				g_rename (fullpath, new_filename);
+				if (g_rename (fullpath, filename) == -1) {
+					g_critical ("%s", g_strerror (errno));
+				}
 			}
-			g_free (new_filename);
+			g_free (filename);
 			g_free (fullpath);
 			f_name = g_dir_read_name (journal_dir);
 		}
@@ -1223,7 +1261,7 @@ void
 tracker_db_manager_remove_temp (void)
 {
 	guint i;
-	gchar *cpath, *new_filename;
+	gchar *cpath, *filename;
 	gchar *directory, *rotate_to = NULL;
 	gsize chunk_size = 0;
 	gboolean do_rotate = FALSE;
@@ -1234,22 +1272,35 @@ tracker_db_manager_remove_temp (void)
 	g_message ("Removing all temp database files");
 
 	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
-		new_filename = g_strdup_printf ("%s.tmp", dbs[i].abs_filename);
+		filename = g_strdup_printf ("%s.tmp", dbs[i].abs_filename);
 		g_message ("  Removing temp database:'%s'",
-		           new_filename);
-		g_unlink (new_filename);
-		g_free (new_filename);
+		           filename);
+		if (g_unlink (filename) == -1) {
+			g_message ("%s", g_strerror (errno));
+		}
+		g_free (filename);
 	}
 
 	cpath = g_strdup (tracker_db_journal_get_filename ());
-	new_filename = g_strdup_printf ("%s.tmp", cpath);
+	filename = g_strdup_printf ("%s.tmp", cpath);
 	g_message ("  Removing temp journal:'%s'",
-	           new_filename);
-	g_unlink (new_filename);
-	g_free (new_filename);
+	           filename);
+	if (g_unlink (filename) == -1) {
+		g_message ("%s", g_strerror (errno));
+	}
+	g_free (filename);
 
 	directory = g_path_get_dirname (cpath);
 	tracker_db_journal_get_rotating (&do_rotate, &chunk_size, &rotate_to);
+	g_free (cpath);
+
+	cpath = g_build_filename (directory, TRACKER_DB_JOURNAL_ONTOLOGY_FILENAME, NULL);
+	filename = g_strdup_printf ("%s.tmp", cpath);
+	if (g_unlink (filename) == -1) {
+		g_message ("%s", g_strerror (errno));
+	}
+	g_free (filename);
+	g_free (cpath);
 
 	dirs[0] = directory;
 	dirs[1] = do_rotate ? rotate_to : NULL;
@@ -1269,9 +1320,11 @@ tracker_db_manager_remove_temp (void)
 				}
 			}
 
-			new_filename = g_build_filename (dirs[i], f_name, NULL);
-			g_unlink (new_filename);
-			g_free (new_filename);
+			filename = g_build_filename (dirs[i], f_name, NULL);
+			if (g_unlink (filename) == -1) {
+				g_message ("%s", g_strerror (errno));
+			}
+			g_free (filename);
 
 			f_name = g_dir_read_name (journal_dir);
 		}
@@ -1281,39 +1334,37 @@ tracker_db_manager_remove_temp (void)
 
 	g_free (rotate_to);
 	g_free (directory);
-	g_free (cpath);
 }
 
 void
 tracker_db_manager_optimize (void)
 {
 	gboolean dbs_are_open = FALSE;
-	guint    i;
+	TrackerDBInterface *iface;
 
 	g_return_if_fail (initialized != FALSE);
 
-	g_message ("Optimizing databases...");
+	g_message ("Optimizing database...");
 
-	g_message ("  Checking DBs are not open");
+	g_message ("  Checking database is not in use");
+
+	iface = tracker_db_manager_get_db_interface ();
 
 	/* Check if any connections are open? */
-	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
-		if (G_OBJECT (dbs[i].iface)->ref_count > 1) {
-			g_message ("  DB:'%s' is still open with %d references!",
-			           dbs[i].name,
-			           G_OBJECT (dbs[i].iface)->ref_count);
+	if (G_OBJECT (iface)->ref_count > 1) {
+		g_message ("  database is still in use with %d references!",
+		           G_OBJECT (iface)->ref_count);
 
-			dbs_are_open = TRUE;
-		}
+		dbs_are_open = TRUE;
 	}
 
 	if (dbs_are_open) {
-		g_message ("  Not optimizing DBs, some are still open with > 1 reference");
+		g_message ("  Not optimizing database, still in use with > 1 reference");
 		return;
 	}
 
 	/* Optimize the metadata database */
-	db_manager_analyze (TRACKER_DB_METADATA);
+	db_manager_analyze (TRACKER_DB_METADATA, iface);
 }
 
 const gchar *
