@@ -89,7 +89,7 @@ static const char imonths[] = {
  * Returns: the first string pointer from those provided which
  * matches, otherwise %NULL.
  *
- * Since: 0.9
+ * Since: 0.10
  **/
 const gchar *
 tracker_coalesce_strip (gint n_values,
@@ -180,7 +180,7 @@ tracker_coalesce (gint n_values,
  * Returns: a newly-allocated string holding the result which should
  * be freed with g_free() when finished with, otherwise %NULL.
  *
- * Since: 0.9
+ * Since: 0.10
  **/
 gchar *
 tracker_merge_const (const gchar *delimiter,
@@ -371,11 +371,11 @@ tracker_text_normalize (const gchar *text,
  *
  * Returns: %TRUE if some bytes were found to be valid, %FALSE otherwise.
  *
- * Since: 0.9
+ * Since: 0.10
  **/
 gboolean
 tracker_text_validate_utf8 (const gchar  *text,
-                            gsize         text_len,
+                            gssize        text_len,
                             GString     **str,
                             gsize        *valid_len)
 {
@@ -907,7 +907,7 @@ my_igetdelim (gchar  **linebuf,
  * can be used to handle embedded %NULL bytes in the line read. Upon
  * failure, -1 is returned.
  *
- * Since: 0.9
+ * Since: 0.10
  **/
 gssize
 tracker_getline (gchar **lineptr,
@@ -936,7 +936,7 @@ tracker_getline (gchar **lineptr,
  * can be used to handle embedded %NULL bytes in the line read. Upon
  * failure, -1 is returned.
  *
- * Since: 0.9
+ * Since: 0.10
  **/
 gssize
 tracker_getline (gchar **lineptr,
@@ -947,3 +947,68 @@ tracker_getline (gchar **lineptr,
 }
 
 #endif /* HAVE_GETLINE */
+
+
+/**
+ * tracker_keywords_parse:
+ * @store: Array where to store the keywords
+ * @keywords: Keywords line to parse
+ *
+ * Parses a keywords line into store, avoiding duplicates and stripping leading
+ * and trailing spaces from keywords. Allowed delimiters are , and ;
+ *
+ * Since: 0.10
+ **/
+void
+tracker_keywords_parse (GPtrArray   *store,
+                        const gchar *keywords)
+{
+	gchar *keywords_d = g_strdup (keywords);
+	char *saveptr, *p;
+	size_t len;
+
+	p = keywords_d;
+	keywords_d = strchr (keywords_d, '"');
+
+	if (keywords_d) {
+		keywords_d++;
+	} else {
+		keywords_d = p;
+	}
+
+	len = strlen (keywords_d);
+	if (keywords_d[len - 1] == '"') {
+		keywords_d[len - 1] = '\0';
+	}
+
+	for (p = strtok_r (keywords_d, ",;", &saveptr); p;
+	     p = strtok_r (NULL, ",;", &saveptr)) {
+		guint i;
+		gboolean found = FALSE;
+		gchar *p_do = g_strdup (p);
+		gchar *p_dup = p_do;
+		guint len = strlen (p_dup);
+
+		if (*p_dup == ' ')
+			p_dup++;
+
+		if (p_dup[len-1] == ' ')
+			p_dup[len-1] = '\0';
+
+		for (i = 0; i < store->len; i++) {
+			const gchar *earlier = g_ptr_array_index (store, i);
+			if (g_strcmp0 (earlier, p_dup) == 0) {
+				found = TRUE;
+				break;
+			}
+		}
+
+		if (!found) {
+			g_ptr_array_add (store, g_strdup (p_dup));
+		}
+
+		g_free (p_do);
+	}
+
+	g_free (keywords_d);
+}
