@@ -495,6 +495,34 @@ extract_tiff (const gchar          *uri,
 			tracker_sparql_builder_object_unvalidated (metadata, md.country);
 		}
 
+		tracker_sparql_builder_predicate (metadata, "mlo:asPostalAddress");
+		tracker_sparql_builder_object_blank_open (metadata); /* PostalAddress */
+
+		tracker_sparql_builder_predicate (metadata, "a");
+		tracker_sparql_builder_object (metadata, "nco:PostalAddress");
+
+		if (md.address) {
+			tracker_sparql_builder_predicate (metadata, "nco:streetAddress");
+			tracker_sparql_builder_object_unvalidated (metadata, md.address);
+		}
+
+		if (md.state) {
+			tracker_sparql_builder_predicate (metadata, "nco:region");
+			tracker_sparql_builder_object_unvalidated (metadata, md.state);
+		}
+
+		if (md.city) {
+			tracker_sparql_builder_predicate (metadata, "nco:locality");
+			tracker_sparql_builder_object_unvalidated (metadata, md.city);
+		}
+
+		if (md.country) {
+			tracker_sparql_builder_predicate (metadata, "nco:country");
+			tracker_sparql_builder_object_unvalidated (metadata, md.country);
+		}
+
+		tracker_sparql_builder_object_blank_close (metadata); /* PostalAddress */
+
 		tracker_sparql_builder_object_blank_close (metadata);
 	}
 
