@@ -49,6 +49,9 @@ class StoreHelper:
         tracker_stats = bus.get_object (cfg.TRACKER_BUSNAME, cfg.TRACKER_STATS_OBJ_PATH)
         self.stats_iface = dbus.Interface (tracker_stats, dbus_interface=cfg.STATS_IFACE)
 
+        tracker_status = bus.get_object (cfg.TRACKER_BUSNAME, cfg.TRACKER_STATUS_OBJ_PATH)
+        self.status_iface = dbus.Interface (tracker_status, dbus_interface=cfg.STATUS_IFACE)
+
     def query (self, query, timeout=5000):
         try:
             return self.resources.SparqlQuery (query, timeout=timeout)
@@ -94,6 +97,14 @@ class StoreHelper:
             self.connect ()
             return self.stats_iface.Get ()
 
+    def wait (self):
+        try:
+            return self.status_iface.Wait ()
+        except dbus.DBusException:
+            self.connect ()
+            return self.status_iface.Wait ()
+
+
     def get_tracker_iface (self):
         return self.resources
 
@@ -115,6 +126,20 @@ class StoreHelper:
             return -1
 
 
+    def ask (self, ask_query):
+        assert ask_query.strip ().startswith ("ASK")
+        result = self.query (ask_query)
+        assert len (result) == 1
+        if result[0][0] == "1":
+            return True
+        elif result[0][0] == "0":
+            return False
+        else:
+            raise Exception ("Something fishy is going on")
+            
+        
+
+        
 class MinerHelper ():
 
     def __init__ (self):
